/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.media.dialog;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.DialogFragment;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.item.Item;
import com.thingspace.cloud.sdk.list.item.Photo;
import com.thingspace.cloud.sdk.list.item.SystemAttributes;
import com.thingspace.media.R;
import com.thingspace.media.listeners.ThingspaceResponseListener;
import com.thingspace.media.tasks.HubbleMediaTask;
import com.thingspace.media.tasks.tags.DeleteTagsTask;
import com.thingspace.media.tasks.tags.GetTagsTask;
import com.thingspace.media.tasks.tags.SetTagsTask;

import java.text.DecimalFormat;

/**
 * AlertDialog to display file information.
 * This class also retrieves, updates and delete tags stored in the cloud for a photo.
 */
public class FileInfoDialog extends DialogFragment implements ThingspaceResponseListener {

    public static final String TAG = "FileInfoDialog";
    private static CloudAPI cloudAPI;
    private static Item item;

    private EditText editTextTags;
    private AlertDialog dialog;
    private ProgressDialog progressDialog;

    private String name;
    private String parentPath;
    private String tags;
    private String mimeType;
    private String createdDate;
    private String width;
    private String height;
    private String size;
    private String uri;

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {

        // Use the Builder class for convenient dialog construction
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.fileinfo, null);
        builder.setView(dialogView);

        loadAttributes();

        ((TextView) dialogView.findViewById(R.id.fileinfo_name)).setText(name);
        ((TextView) dialogView.findViewById(R.id.fileinfo_mime)).setText(mimeType);
        if (size != null) {
            ((TextView) dialogView.findViewById(R.id.fileinfo_size)).setText(getFileSize(Long.parseLong(size)));
        }
        ((TextView) dialogView.findViewById(R.id.fileinfo_date)).setText(retrieveDate(createdDate));
        ((TextView) dialogView.findViewById(R.id.fileinfo_resolution)).setText(width + " X " + height);

        editTextTags = (EditText) dialogView.findViewById(R.id.editText_tags);
        //editTextTags.setText(tags);
        editTextTags.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                editTextTags.setCursorVisible(true);
            }
        });

        builder.setPositiveButton(R.string.update,
            new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {

                }
        });

        builder.setNegativeButton(R.string.close,
            new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    FileInfoDialog.this.getDialog().cancel();
                }
        });

        // Create the AlertDialog object and return it
        dialog = builder.create();
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {

            @Override
            public void onShow(DialogInterface dialogInterface) {

                Button b = dialog.getButton(AlertDialog.BUTTON_POSITIVE);
                b.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View view) {
                        if (TextUtils.isEmpty(editTextTags.getText().toString())) {
                            /**
                             * DeleteTagsTask calls the Cloud SDK API method to delete tags stored in the cloud for a file.
                             */
                            DeleteTagsTask task = new DeleteTagsTask(getActivity(), FileInfoDialog.this, cloudAPI, uri);
                            task.execute();

                            progressDialog = ProgressDialog.show(getActivity(), "", "Deleting tags", true);
                            progressDialog.setCancelable(true);
                        } else {
                            if (!editTextTags.getText().toString().equals(tags)) {
                                /**
                                 * SetTagsTask calls the Cloud SDK API method to update tags for a file.
                                 */
                                SetTagsTask task = new SetTagsTask(getActivity(), FileInfoDialog.this, cloudAPI, uri, editTextTags.getText().toString());
                                task.execute();

                                progressDialog = ProgressDialog.show(getActivity(), "", "Updating tags", true);
                                progressDialog.setCancelable(true);
                            } else {
                                Toast.makeText(getActivity(), "No Update", Toast.LENGTH_SHORT).show();
                            }
                        }
                    }
                });
            }
        });

        dialog.show();
        getTags();
        return dialog;
    }

    private String retrieveDate(String date) {
        if (date != null && date.contains("T")) {
            return date.substring(0, date.indexOf("T"));
        }
        return null;
    }

    private String getFileSize(long size) {
        if (size <= 0)
            return "0";
        final String[] units = new String[]{"B", "KB", "MB", "GB", "TB"};
        int digitGroups = (int) (Math.log10(size) / Math.log10(1024));
        return new DecimalFormat("#.##").format(size / Math.pow(1024, digitGroups)) + " " + units[digitGroups];
    }

    private void loadAttributes() {
        if (item instanceof Photo) {
            Photo photo = (Photo) item;
            name = photo.getName();
            parentPath = photo.getParentPath();
            mimeType = photo.getMimeType();
            size = String.valueOf(photo.getSize());
            createdDate = photo.getVersionCreated();
            width = String.valueOf(photo.getWidth());
            height = String.valueOf(photo.getHeight());
            tags = photo.getTags();
            uri = photo.getUri();
        } else {
            SystemAttributes systemAttributes = (SystemAttributes) item;
            name = systemAttributes.getName();
            parentPath = systemAttributes.getParentPath();
            mimeType = systemAttributes.getMimeType();
            size = String.valueOf(systemAttributes.getSize());
            createdDate = systemAttributes.getVersionCreated();
            width = String.valueOf(systemAttributes.getWidth());
            height = String.valueOf(systemAttributes.getHeight());
            tags = systemAttributes.getTags();
            uri = systemAttributes.getUri();
        }
    }

    private void getTags() {
        String path = parentPath + "/" + name;

        /**
         * GetTagsTask calls Cloud SDK method to retrieve tags stored in the cloud for a file.
         * Tags can also be retrieved using other api calls that returns a list of files.
         */
        GetTagsTask task = new GetTagsTask(getActivity(), this, cloudAPI, path);
        task.execute();

        progressDialog = ProgressDialog.show(getActivity(), "", "Retrieving tags", true);
        progressDialog.setCancelable(true);
    }

    @Override
    public void onResponse(int taskCode, int errorCode, Object response) {
        progressDialog.dismiss();

        if (response == null) {
            return;
        }

        if (taskCode == HubbleMediaTask.TASK_DELETE_TAGS) {
            if ((boolean) response) {
                tags = "";
                editTextTags.setText(tags);
                Toast.makeText(getActivity(), "Tags Deleted", Toast.LENGTH_SHORT).show();
            }
        } else if ((taskCode == HubbleMediaTask.TASK_SET_TAGS)) {
            if (response instanceof String) {
                tags = (String) response;
                editTextTags.setText(tags);
                Toast.makeText(getActivity(), "Tags Updated", Toast.LENGTH_SHORT).show();
            }
        } else if (taskCode == HubbleMediaTask.TASK_GET_TAGS) {
            if (response instanceof String) {
                tags = (String) response;
                editTextTags.setText(tags);
            }
        }
    }

    public static void setItem(Item aItem) {
        item = aItem;
    }

    public static void setCloudAPI (CloudAPI aCloudAPI) {
        cloudAPI = aCloudAPI;
    }
}
