/*
 * Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.
 */

package com.thingspace.files.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.thingspace.cloud.sdk.CloudAPI;
import com.thingspace.cloud.sdk.list.ItemList;
import com.thingspace.cloud.sdk.list.item.MetadataItem;
import com.thingspace.files.R;
import com.thingspace.files.cache.ImageCache;
import com.thingspace.files.cache.ImageFetcher;

/**
 * MetadataGridAdapter to inflate grid items.
 */
public class MetadataGridAdapter extends BaseAdapter {

    private static final String MEDIA_TYPE_AUDIO = "audio";
    private static final String MEDIA_TYPE_VIDEO = "video";
    private static final String MEDIA_TYPE_IMAGE = "image";
    private static ImageFetcher staticImageFetcher;
    private Context context;
    private int layoutID;
    private ItemList<MetadataItem> metadataResponse;
    private CloudAPI cloudAPI;

    /**
     * Inflate grid items and retrieve content tokens from SQLite to display thumbnails.
     *
     * @param context          context to initiate SimpleCursorAdapter and invoke inflater.
     * @param cloudAPI         CloudAPI object to access cloud storage
     * @param layout           resource identifier of a layout file that defines the views for this list item. The layout file should include at least those named views defined in "to"
     * @param metadataResponse List of file/folder response for cloud folder
     */
    public MetadataGridAdapter(Context context, CloudAPI cloudAPI, int layout, ItemList<MetadataItem> metadataResponse) {
        this.context = context;
        this.cloudAPI = cloudAPI;
        this.layoutID = layout;
        this.metadataResponse = metadataResponse;
    }

    /**
     * @param position
     * @param convertView
     * @param parent
     * @return
     */
    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        View row = convertView;
        ViewHolder viewHolder;
        String name = "";
        MetadataItem item = getItem(position);

        if (row == null) {
            row = LayoutInflater.from(context).inflate(layoutID, parent, false);
            viewHolder = new ViewHolder();
            viewHolder.image = (ImageView) row.findViewById(R.id.image);
            viewHolder.name = (TextView) row.findViewById(R.id.name);
            row.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) row.getTag();
        }

        if (item == null) {
            return row;
        }

        // Android recycles the views and reuses them, hence the background image set in getImageFetcher stays when you scroll, and it appears in other thumbs.
        // This is a temporary solution, this needs to be handled in the if below.
        viewHolder.image.setBackgroundDrawable(null);

        name = item.getName();
        if (item.getContentToken() != null || (item.getMimeType() != null && !item.getMimeType().equalsIgnoreCase("directory"))) {

            if (item.getMimeType().contains(MEDIA_TYPE_AUDIO)) {
                viewHolder.image.setImageResource(R.drawable.music_icon);
            } else if (item.getMimeType().contains(MEDIA_TYPE_VIDEO)) {
                viewHolder.image.setImageResource(R.drawable.videos_icon);
            } else if (item.getMimeType().contains(MEDIA_TYPE_IMAGE)) {
                viewHolder.image.setImageResource(R.drawable.thumb_placeholder);
            } else {
                viewHolder.image.setImageResource(R.drawable.file_icon);
            }

            if (item.getContentToken() != null && (item.getMimeType().contains(MEDIA_TYPE_VIDEO) || item.getMimeType().contains(MEDIA_TYPE_IMAGE))) {
                getImageFetcher().loadImage(item.getContentToken(), viewHolder.image, cloudAPI);
            }

        } else {
            viewHolder.image.setImageResource(R.drawable.folder_icon);
        }

        viewHolder.name.setText(name);
        return row;
    }

    /**
     * @param position
     * @return
     */
    @Override
    public long getItemId(int position) {
        return position;
    }

    /**
     * @param position
     * @return
     */
    @Override
    public MetadataItem getItem(int position) {
        return metadataResponse.getItem(position);
    }

    /**
     * @return
     */
    @Override
    public int getCount() {
        return metadataResponse.getCount();
    }

    public ImageFetcher getImageFetcher() {
        if (staticImageFetcher == null) {
            ImageCache.ImageCacheParams cacheParams = new ImageCache.ImageCacheParams(context, "thumbs");
            cacheParams.setMemCacheSizePercent(context, 0.5f);

            staticImageFetcher = new ImageFetcher(context, context.getResources().getDimensionPixelSize(R.dimen.photo_item_size));
            staticImageFetcher.addImageCache(cacheParams);
        }
        return staticImageFetcher;
    }

    /**
     * View holder class for Image and Text.
     */
    static class ViewHolder {
        ImageView image;
        TextView name;
    }
}
